//------------------------------------------------------------------------------------------------
//
//   SG Craft - Computercraft Interface Peripheral
//
//------------------------------------------------------------------------------------------------

package gcewing.sg.cc;

import com.google.common.base.Joiner;

import net.minecraft.tileentity.*;
import net.minecraft.world.*;
import net.minecraftforge.common.util.*;

import dan200.computercraft.api.lua.*;
import dan200.computercraft.api.peripheral.*;

import gcewing.sg.*;

public class CCSGPeripheral implements IPeripheral {

	static CCMethod[] methods = {
	
		new SGMethod("stargateState") {
			Object[] call(SGInterfaceTE te, Object[] args) {
				SGBaseTE.CIStargateState result = SGBaseTE.ciStargateState(te);
				return new Object[] {result.state, result.chevrons};
			}
		},
		
		new SGMethod("energyAvailable") {
			Object[] call(SGInterfaceTE te, Object[] args) {
				return new Object[] {SGBaseTE.ciEnergyAvailable(te)};
			}
		},
		
		new SGMethod("energyToDial", 1) {
			Object[] call(SGInterfaceTE te, Object[] args) {
				return new Object[] {SGBaseTE.ciEnergyToDial(te, (String)args[0])};
			}
		},
		
		new SGMethod("localAddress") {
			Object[] call(SGInterfaceTE te, Object[] args) {
				return new Object[] {SGBaseTE.ciLocalAddress(te)};
			}
		},
		
		new SGMethod("remoteAddress") {
			Object[] call(SGInterfaceTE te, Object[] args) {
				return new Object[] {SGBaseTE.ciRemoteAddress(te)};
			}
		},
		
		new SGMethod("dial", 1) {
			Object[] call(SGInterfaceTE te, Object[] args) {
				SGBaseTE.ciDial(te, (String)args[0]);
				return null;
			}
		},
		
		new SGMethod("disconnect") {
			Object[] call(SGInterfaceTE te, Object[] args) {
				SGBaseTE.ciDisconnect(te);
				return null;
			}
		},
		
		new SGMethod("irisState") {
			Object[] call(SGInterfaceTE te, Object[] args) {
				return new Object[] {SGBaseTE.ciIrisState(te)};
			}
		},
		
		new SGMethod("openIris") {
			Object[] call(SGInterfaceTE te, Object[] args) {
				SGBaseTE.ciOpenIris(te);
				return null;
			}
		},
		
		new SGMethod("closeIris") {
			Object[] call(SGInterfaceTE te, Object[] args) {
				SGBaseTE.ciCloseIris(te);
				return null;
			}
		},

	};
	
	World worldObj;
	int xCoord, yCoord, zCoord;
	
	public CCSGPeripheral(TileEntity te) {
		worldObj = te.getWorldObj();
		xCoord = te.xCoord;
		yCoord = te.yCoord;
		zCoord = te.zCoord;
		//System.out.printf("CCInterfaceTE: Created\n");
	}
	
//	SGBaseTE getBaseTE() {
//		return SGBaseTE.get(worldObj, xCoord, yCoord + 1, zCoord);
//	}
	
	CCInterfaceTE getInterfaceTE() {
		TileEntity te = worldObj.getTileEntity(xCoord, yCoord, zCoord);
		if (te instanceof CCInterfaceTE)
			return (CCInterfaceTE)te;
		else
			throw new IllegalArgumentException("Stargate interface failed internal diagnostics");
	}
	
	public String getType() {
		return "stargate";
	}
	
	public String[] getMethodNames() {
		String[] result = new String[methods.length];
		for (int i = 0; i < methods.length; i++)
			result[i] = methods[i].name;
		return result;
	}
	
	public Object[] callMethod(IComputerAccess cpu, ILuaContext ctx, int method, Object[] args)
		throws LuaException, InterruptedException
	{
		if (method >= 0 && method < methods.length)
			return CCMethodQueue.instance.invoke(cpu, ctx, this, methods[method], args);
		else
			throw new LuaException(String.format("Invalid method index %s", method));
	}
	
	public void attach(IComputerAccess cpu) {
		System.out.printf("CCSGPeripheral.attach: to %s\n", cpu);
		CCInterfaceTE te = getInterfaceTE();
		if (te != null)
			te.attachedComputers.add(cpu);
	}
	
	public void detach(IComputerAccess cpu) {
		System.out.printf("CCSGPeripheral.detach: from %s\n", cpu);
		CCInterfaceTE te = getInterfaceTE();
		if (te != null)
			te.attachedComputers.remove(cpu);
	}
	
	public boolean equals(IPeripheral other) {
		return this == other;
	}

}

//------------------------------------------------------------------------------------------------

abstract class SGMethod extends CCMethod {

	public SGMethod(String name) {
		super(name);
	}
	
	public SGMethod(String name, int nargs) {
		super(name, nargs);
	}

	@Override
	Object[] call(IComputerAccess cpu, ILuaContext ctx, Object target, Object[] args) {
		SGInterfaceTE te = ((CCSGPeripheral)target).getInterfaceTE();
		return call(te, args);
	}
	
	abstract Object[] call(SGInterfaceTE te, Object[] args);

}

