//------------------------------------------------------------------------------------------------
//
//	 SG Craft - Open Computers Interface Tile Entity
//
//------------------------------------------------------------------------------------------------

package gcewing.sg.oc;

import java.util.*;
import net.minecraft.nbt.*;
import net.minecraft.world.*;
import net.minecraft.tileentity.*;

import li.cil.oc.api.Network;
import li.cil.oc.api.machine.Value;
import li.cil.oc.api.network.Arguments;
import li.cil.oc.api.network.Context;
import li.cil.oc.api.network.Callback;
import li.cil.oc.api.network.Environment;
import li.cil.oc.api.network.Message;
import li.cil.oc.api.network.Node;
import li.cil.oc.api.network.Visibility;

import gcewing.sg.*;

public class OCInterfaceTE extends SGInterfaceTE implements IComputerInterface, Environment {

//	Set<IComputerAccess> attachedComputers = new HashSet<IComputerAccess>();

	public OCInterfaceTE() {
		node = Network.newNode(this, Visibility.Network)
			.withComponent("stargate", Visibility.Neighbors)
			.create();
	}
	
//	SGBaseTE getBaseTE() {
//		SGBaseTE te = SGBaseTE.get(worldObj, xCoord, yCoord + 1, zCoord);
//		if (te != null)
//			return te;
//		throw new IllegalArgumentException("No stargate connected to interface");
//	}

	@Callback
	public Object[] stargateState(Context ctx, Arguments args) {
		SGBaseTE.CIStargateState result = SGBaseTE.ciStargateState(this);
		return new Object[]{result.state, result.chevrons};
	}
	
	@Callback
	public Object[] energyAvailable(Context ctx, Arguments args) {
		return new Object[]{SGBaseTE.ciEnergyAvailable(this)};
	}
	
	@Callback
	public Object[] energyToDial(Context ctx, Arguments args) {
		return new Object[]{SGBaseTE.ciEnergyToDial(this, args.checkString(0))};
	}
	
	@Callback
	public Object[] localAddress(Context ctx, Arguments args) {
		return new Object[]{SGBaseTE.ciLocalAddress(this)};
	}
	
	@Callback
	public Object[] remoteAddress(Context ctx, Arguments args) {
		return new Object[]{SGBaseTE.ciRemoteAddress(this)};
	}
	
	@Callback
	public Object[] dial(Context ctx, Arguments args) {
		SGBaseTE.ciDial(this, args.checkString(0));
		return null;
	}
	
	@Callback
	public Object[] disconnect(Context ctx, Arguments args) {
		SGBaseTE.ciDisconnect(this);
		return null;
	}
	
	@Callback
	public Object[] irisState(Context ctx, Arguments args) {
		return new Object[]{SGBaseTE.ciIrisState(this)};
	}
	
	@Callback
	public Object[] openIris(Context ctx, Arguments args) {
		SGBaseTE.ciOpenIris(this);
		return null;
	}
	
	@Callback
	public Object[] closeIris(Context ctx, Arguments args) {
		SGBaseTE.ciCloseIris(this);
		return null;
	}

	// -------------------------- Environment --------------------------

    /**
     * This must be set in subclasses to the node that is used to represent
     * this tile entity.
     * <p/>
     * You must only create new nodes using the factory method in the network
     * API, {@link li.cil.oc.api.Network#newNode(Environment, Visibility)}.
     * <p/>
     * For example:
     * <pre>
     * // The first parameters to newNode is the host() of the node, which will
     * // usually be this tile entity. The second one is it's reachability,
     * // which determines how other nodes in the same network can query this
     * // node. See {@link li.cil.oc.api.network.Network#nodes(li.cil.oc.api.network.Node)}.
     * node = Network.newNode(this, Visibility.Network)
     *       // This call allows the node to consume energy from the
     *       // component network it is in and act as a consumer, or to
     *       // inject energy into that network and act as a producer.
     *       // If you do not need energy remove this call.
     *       .withConnector()
     *       // This call marks the tile entity as a component. This means you
     *       // can mark methods in it using the {@link li.cil.oc.api.network.Callback}
     *       // annotation, making them callable from user code. The first
     *       // parameter is the name by which the component will be known in
     *       // the computer, in this case it could be accessed as
     *       // <tt>component.example</tt>. The second parameter is the
     *       // component's visibility. This is like the node's reachability,
     *       // but only applies to computers. For example, network cards can
     *       // only be <em>seen</em> by the computer they're installed in, but
     *       // can be <em>reached</em> by all other network cards in the same
     *       // network. If you do not need callbacks remove this call.
     *       .withComponent("example", Visibility.Neighbors)
     *       // Finalizes the construction of the node and returns it.
     *       .create();
     * </pre>
     */
	protected Node node;

	protected boolean addedToNetwork = false;

	@Override
	public Node node() {
		return node;
	}

	@Override
	public void onConnect(final Node node) {
		// This is called when the call to Network.joinOrCreateNetwork(this) in
		// updateEntity was successful, in which case `node == this`.
		// This is also called for any other node that gets connected to the
		// network our node is in, in which case `node` is the added node.
		// If our node is added to an existing network, this is called for each
		// node already in said network.
	}

	@Override
	public void onDisconnect(final Node node) {
		// This is called when this node is removed from its network when the
		// tile entity is removed from the world (see onChunkUnload() and
		// invalidate()), in which case `node == this`.
		// This is also called for each other node that gets removed from the
		// network our node is in, in which case `node` is the removed node.
		// If a net-split occurs this is called for each node that is no longer
		// connected to our node.
	}

	@Override
	public void onMessage(final Message message) {
		// This is used to deliver messages sent via node.sendToXYZ. Handle
		// messages at your own discretion. If you do not wish to handle a
		// message you should *not* throw an exception, though.
	}

	// ----------------------------------------------------------------------- //

	@Override
	public void updateEntity() {
		super.updateEntity();
		// On the first update, try to add our node to nearby networks. We do
		// this in the update logic, not in validate() because we need to access
		// neighboring tile entities, which isn't possible in validate().
		// We could alternatively check node != null && node.network() == null,
		// but this has somewhat better performance, and makes it clearer.
		if (!addedToNetwork) {
			addedToNetwork = true;
			Network.joinOrCreateNetwork(this);
		}
	}

	@Override
	public void onChunkUnload() {
		super.onChunkUnload();
		// Make sure to remove the node from its network when its environment,
		// meaning this tile entity, gets unloaded.
		if (node != null) node.remove();
	}

	@Override
	public void invalidate() {
		super.invalidate();
		// Make sure to remove the node from its network when its environment,
		// meaning this tile entity, gets unloaded.
		if (node != null) node.remove();
	}

	// ----------------------------------------------------------------------- //

	@Override
	public void readFromNBT(final NBTTagCompound nbt) {
		super.readFromNBT(nbt);
		// The host check may be superfluous for you. It's just there to allow
		// some special cases, where getNode() returns some node managed by
		// some other instance (for example when you have multiple internal
		// nodes in this tile entity).
		if (node != null && node.host() == this) {
			// This restores the node's address, which is required for networks
			// to continue working without interruption across loads. If the
			// node is a power connector this is also required to restore the
			// internal energy buffer of the node.
			node.load(nbt.getCompoundTag("oc:node"));
		}
	}

	@Override
	public void writeToNBT(final NBTTagCompound nbt) {
		super.writeToNBT(nbt);
		// See readFromNBT() regarding host check.
		if (node != null && node.host() == this) {
			final NBTTagCompound nodeNbt = new NBTTagCompound();
			node.save(nodeNbt);
			nbt.setTag("oc:node", nodeNbt);
		}
	}


	// -------------------------- IComputerInterface --------------------------

	public void postEvent(TileEntity source, String name, Object... args) {
		System.out.printf("OCInterfaceTE.postEvent: %s to %s\n", name, node);
//		World world = source.getWorldObj();
//		OCSGEventInfo info = new OCSGEventInfo();
//		info.world = world.getWorldInfo().getWorldName();
//		info.dimension = world.provider.dimensionId;
//		info.x = source.xCoord;
//		info.y = source.yCoord;
//		info.z = source.zCoord;
//		for (int i = 0; i < attrs.length - 1; i += 2)
//			info.add((String)attrs[i], attrs[i + 1]);
		if (node != null)
			node.sendToReachable("computer.signal", prependArgs(name, args));
	}
	
}

//------------------------------------------------------------------------------------------------

//class OCSGEventInfo implements Value {
//
//	public String world;
//	public int dimension;
//	public int x, y, z;
//	NBTTagCompound fields = new NBTTagCompound();
//	
//	public void add(String name, Object value) {
//		if (value instanceof Integer)
//			fields.setInteger(name, (Integer)value);
//		else if (value instanceof String)
//			fields.setString(name, (String)value);
//	}
//	
//	public void load(NBTTagCompound nbt) {
//		world = nbt.getString("world");
//		dimension = nbt.getInteger("dimension");
//		x = nbt.getInteger("x");
//		y = nbt.getInteger("y");
//		z = nbt.getInteger("z");
//		fields = nbt.getCompoundTag("fields");
//	}
//	
//	public void save(NBTTagCompound nbt) {
//		nbt.setString("world", world);
//		nbt.setInteger("dimension", dimension);
//		nbt.setInteger("x", x);
//		nbt.setInteger("y", y);
//		nbt.setInteger("z", z);
//		nbt.setTag("fields", fields);
//	}
//	
//	public Object apply(Context context, Arguments arguments) {return null;}
//	public void unapply(Context context, Arguments arguments) {}
//	public Object[] call(Context context, Arguments arguments) {return null;}
//	public void dispose(Context context) {}
//	
//	static Object[] olist(Object... args) {
//		return args;
//	}
//	
//	@Callback
//	public Object[] world() {return olist(world);}
//	
//	@Callback
//	public Object[] dimemsion() {return olist(dimension);}
//	
//	@Callback
//	public Object[] x() {return olist(x);}
//	
//	@Callback
//	public Object[] y() {return olist(y);}
//	
//	@Callback
//	public Object[] z() {return olist(z);}
//	
//	@Callback
//	public Object[] oldState() {return olist(fields.getString("oldState"));}
//	
//	@Callback
//	public Object[] newState() {return olist(fields.getString("newState"));}
//	
//	@Callback
//	public Object[] address() {return olist(fields.getString("address"));}
//	
//	@Callback
//	public Object[] chevron() {return olist(fields.getInteger("chevron"));}
//	
//	@Callback
//	public Object[] symbol() {return olist(fields.getString("symbol"));}
//	
//}
